unit PowHelp;

{$C MOVEABLE PRELOAD PERMANENT}

interface

uses WinTypes, WinProcs, Strings, OWindows, ODialogs;


{ Global Help wrapper stuff }
const
  HelpOpen: Boolean = False;
  HelpFID: PChar = 'POWERD.HLP';
  HelpWnd: HWND = 0;

procedure DoHelp(wCommand: Word; dwData: LongInt);
procedure CloseHelp;

const
  idCadre   = 101;
  idIcon1   = 102;
  idIcon2   = 103;
  idMoreKey = 104;

type
  PCreditWindow = ^TCreditWindow;
  TCreditWindow = object(TWindow)
    Bitmap: HBitmap;
    BitSize: TBitmap;
    ScrollUnit: Integer;
    ScrollRate: Integer;
    ScrollPos: Integer;
    FontHeight: Integer;
    constructor Init(AParent: PWindowsObject; ABitmapName: PChar);
    destructor Done; virtual;
    function  GetClassName: PChar; virtual;
    procedure GetWindowClass(var WC: TWndClass); virtual;
    procedure SetupWindow; virtual;         { First place HWindow is valid }
    procedure WMDestroy(var Msg: TMessage); { Last place HWindow is valid  }
      virtual wm_First + wm_Destroy;
    procedure Paint(DC: HDC; var PS: TPaintStruct); virtual;
    procedure ShowCredits; virtual;
    procedure WriteLine(DC: HDC; Y: Word; const S: array of Char); virtual;
    procedure WMTimer(var Msg: TMessage);
      virtual wm_First + wm_Timer;
  end;


  PAboutBox = ^TAboutBox;
  TAboutBox = object(TDialog)
    Title: PChar;
    CreditWindow: PCreditWindow;
    constructor Init(AParent: PWindowsObject;
                     ATitle, ABitmapName: PChar);
    destructor Done; virtual;
    procedure SetupWindow; virtual;
    function  GetResName: PChar; virtual;
    procedure InitCreditWindow(ABitmapName: PChar); virtual;
    procedure ShowCredits(var Msg: TMessage);
      virtual id_First + idMoreKey;
  end;

implementation

procedure DoHelp;
begin
  if HelpWnd = 0 then
    HelpWnd := Application^.MainWindow^.HWindow;
  if WinHelp(HelpWnd,HelpFID,wCommand,dwData) then
    if not HelpOpen then
      HelpOpen := true;
end;

procedure CloseHelp;
begin
  if HelpOpen then
    WinHelp(HelpWnd, HelpFID, help_Quit, 0);
end;

constructor TCreditWindow.Init(AParent: PWindowsObject;
                               ABitmapName: PChar);
var
  DC: HDC;
  OldFont: HFont;
  TM: TTextMetric;
begin
  inherited Init(AParent, nil);
  Attr.Style := ws_Child or ws_Visible;
  Bitmap := LoadBitmap(HInstance, ABitmapName);
  if Bitmap = 0 then
  begin
    Status := em_InvalidWindow;
    Exit;
  end;
  GetObject(Bitmap, SizeOf(BitSize), @BitSize);
  ScrollPos := 0;
  DC := GetDC(0);
  ScrollUnit := 2;
  ScrollRate := 80;
  OldFont := SelectObject(DC, GetStockObject(ANSI_VAR_FONT));
  GetTextMetrics(DC, TM);
  FontHeight := TM.tmHeight + TM.tmExternalLeading + 5;
  SelectObject(DC, Oldfont);
  ReleaseDC(0, DC);
end;

destructor TCreditWindow.Done;
begin
  inherited Done;
  DeleteObject(Bitmap);
end;

function TCreditWindow.GetClassName: PChar;
begin
  GetClassName := 'PowerAboutCredits';
end;

procedure TCreditWindow.GetWindowClass(var WC: TWndClass);
begin
  inherited GetWindowClass(WC);
  WC.Style := cs_ByteAlignWindow;
  WC.hbrBackground := GetStockObject(Black_Brush);
end;

procedure TCreditWindow.SetupWindow;
begin
  inherited SetupWindow;
  SetWindowPos(HWindow, 0, 0, 0, BitSize.bmWidth, BitSize.bmHeight,
               swp_NoMove or swp_NoZOrder or swp_NoActivate or swp_NoRedraw);
end;

procedure TCreditWindow.WMDestroy(var Msg: TMessage);
begin
  if ScrollPos <> 0 then { We're scrolling and need to kill the timer }
  begin
    KillTimer(HWindow, 1);
    ScrollPos := 0;
  end;
  inherited WMDestroy(Msg);
end;

const
  CreditText: array[1..18] of PChar = (
    '',
    'POWER DESIGNER',
    '',
    'Version 1.30',
    '',
    'A Graphical Package for Editing',
    '',
    'and Simulating Electric Networks',
    '',
    'Programmed by :',
    'Lotfi BAGHLI',
    '',
    'February 1996',
    '',
    'Copyright 1996',
    'All rights reserved',
    '',
    '');

procedure TCreditWindow.Paint(DC: HDC; var PS: TPaintStruct);
var
  R: TRect;
  FirstLine, LastLine: Integer;
  S: array [0..50] of Char;
  CryptMark: Longint;
  Y: Integer;

  procedure DrawBitmap(Y: Integer);
  var
    MemDC: HDC;
    OldBits: HBitmap;
  begin
    MemDC:= CreateCompatibleDC(DC);
    OldBits := SelectObject(MemDC, Bitmap);
    BitBlt(DC, 0, Y, Attr.W, Attr.H, MemDC, 0, 0, srcCopy);
    SelectObject(MemDC, OldBits);
    DeleteDC(MemDC);
  end;

begin
  SaveDC(DC);
  SetViewportOrg(DC, 0, -ScrollPos);
  OffsetRect(PS.rcPaint, 0, ScrollPos);
  with R do
  begin
    Left := 0;
    Top := 0;
    Right := Attr.W;
    Bottom := Attr.H;
  end;
  if Bool(IntersectRect(R, PS.rcPaint, R)) then
  begin
    DrawBitmap(0);
    with PS.rcPaint do
    begin
      if (R.Top < Top) and (R.Bottom > Top) then Top := R.Bottom;
      if (R.Top < Bottom) and (R.Bottom > Bottom) then Bottom := R.Top;
      if Top > Bottom then Top := Bottom;
    end;
  end;
  if ScrollPos > 0 then    { we're scrolling }
  begin
    FirstLine := (PS.rcPaint.Top - Attr.H) div FontHeight;
    if FirstLine < 0 then FirstLine := 0;
    if FirstLine < High(CreditText) then
    begin                             { we have text to draw }
      SetTextAlign(DC, TA_Center);
      SetBkColor(DC, 0);
      SetTextColor(DC, RGB($ff,$ff,$ff));
      LastLine := (PS.rcPaint.Bottom - Attr.H) div FontHeight;
      for Y := FirstLine to LastLine do
        if Y < High(CreditText) then
          WriteLine(DC, Y*FontHeight + Attr.H, CreditText[Y + 1]^);
    end;
                            { Paint second image of bitmap at bottom }
    if PS.rcPaint.Bottom > (Attr.H+FontHeight*High(CreditText)) then
      DrawBitmap(Attr.H + FontHeight * High(CreditText));
  end;
  RestoreDC(DC, -1);
end;

procedure TCreditWindow.ShowCredits;
begin
  SetTimer(HWindow, 1, ScrollRate, nil);
end;

procedure TCreditWindow.WriteLine(DC: HDC; Y: Word; const S: array of Char);
begin
  TextOut(DC, Attr.W div 2, Y, @S, StrLen(@S));
end;

procedure TCreditWindow.WMTimer(var Msg: TMessage);
begin
  Inc(ScrollPos, ScrollUnit);
  { Check to see if it's time to stop scrolling }
  if ScrollPos > Attr.H + FontHeight * High(CreditText) then
  begin
    ScrollPos := 0;
    KillTimer(HWindow, 1);
    InvalidateRect(HWindow, nil, False);
  end
  else
    ScrollWindow(HWindow, 0, -ScrollUnit, nil, nil);
  UpdateWindow(HWindow);
end;

constructor TAboutBox.Init(AParent: PWindowsObject;
                           ATitle, ABitmapName: PChar);
begin
  inherited Init(AParent, GetResName);
  Title := StrNew(ATitle);
  InitCreditWindow(ABitmapName);
end;

destructor TAboutBox.Done;
begin
  inherited Done;
  if Title <> nil then
    StrDispose(Title);
end;

procedure TAboutBox.SetupWindow;
var
  RDialog,R,RBitWnd,RCadre,RIcon,RMoreKey,ROk: TRect;
  XW, X8, Y8: Integer;
  DC: HDC;
begin
  inherited SetupWindow;
  SetWindowText(HWindow, Title);
  DC := GetDC(HWindow);
  X8 := GetDeviceCaps(DC,LogPixelsX) div 8;   { 1/8 inch }
  Y8 := GetDeviceCaps(DC,LogPixelsY) div 8;
  ReleaseDC(HWindow, DC);
  GetClientRect(GetDlgItem(HWindow, idCadre), RCadre);
  GetClientRect(GetDlgItem(HWindow, idIcon1), RIcon);
  GetClientRect(GetDlgItem(HWindow, idMoreKey), RMoreKey);
  GetClientRect(GetDlgItem(HWindow, idOK), ROk);
  GetClientRect(CreditWindow^.HWindow, RBitWnd);
  RCadre.Top := Y8;
  RCadre.Left := X8;
  if RCadre.Right < RBitWnd.Right + 2*X8 then
    RCadre.Right := RBitWnd.Right + 2*X8;
  if RCadre.Bottom < RBitWnd.Bottom + 2*Y8 then
    RCadre.Bottom := RBitWnd.Bottom + 2*Y8;

  with  RDialog do
  begin
    GetWindowRect(HWindow, RDialog);
    GetClientRect(HWindow, R);
    Right := Right - Left - R.Right;
    Bottom := Bottom - Top - R.Bottom;
    Right := Right + X8 + RCadre.Right + X8;   { 1/8 inch margins }
    Bottom := Bottom + Y8 + RCadre.Bottom
                     + 2*Y8 +
                     + Y8 + ROk.Bottom + Y8;
    if Parent <> nil then
    begin
      GetWindowRect(Parent^.HWindow, R);
        { Center dialog in parent's window }
      Left := R.Left + (R.Right - R.Left) div 2 - Right div 2;
      Top := R.Top + (R.Bottom - R.Top) div 2 - Bottom div 2;
    end;
    SetWindowPos(HWindow, 0, Left, Top, Right, Bottom,
                 swp_NoActivate or swp_NoZOrder);
  end;
  with RCadre do
  begin
    SetWindowPos(GetDlgItem(HWindow, idCadre), 0, Left, Top,
                 Right, Bottom, swp_NoActivate or swp_NoZOrder);
    SetWindowPos(CreditWindow^.HWindow, 0,  Left + X8, Top + Y8, 0, 0,
                 swp_NoActivate or swp_NoSize or swp_NoZOrder);
  end;

  GetClientRect(HWindow, R);
  XW:=R.Right div 4;
  with RIcon do
    begin
    SetWindowPos(GetDlgItem(HWindow, idICon1), 0,
                 XW div 2 - Right div 2,
                 RCadre.Bottom + 3*Y8, 0, 0,
                 swp_NoActivate or swp_NoZOrder or swp_NoSize);
    SetWindowPos(GetDlgItem(HWindow, idICon2), 0,
                 3*XW+XW div 2 - Right div 2,
                 RCadre.Bottom + 3*Y8, 0, 0,
                 swp_NoActivate or swp_NoZOrder or swp_NoSize);
    end;
  with RMoreKey do
    SetWindowPos(GetDlgItem(HWindow, idMoreKey), 0,
                 2*XW+XW div 2 - Right div 2,
                 RCadre.Bottom + 3*Y8, 0, 0,
                 swp_NoActivate or swp_NoZOrder or swp_NoSize);
  with ROk do
    SetWindowPos(GetDlgItem(HWindow, idOk), 0,
                 1*XW+XW div 2 - Right div 2,
                 RCadre.Bottom + 3*Y8, 0, 0,
                 swp_NoActivate or swp_NoZOrder or swp_NoSize);
end;

function TAboutBox.GetResName: PChar;
begin
  GetResName := 'AboutDlg16';
end;

procedure TAboutBox.InitCreditWindow(ABitmapName: PChar);
begin
  CreditWindow := New(PCreditWindow, Init(@Self, ABitmapName));
end;

procedure TAboutBox.ShowCredits(var Msg: TMessage);
begin
  CreditWindow^.ShowCredits;
end;


End.