{
 StatusLine pour Power V1.20 TDesigner MDIWindow
  11 mars 1994
}
unit Ruban;

{$C MOVEABLE PRELOAD PERMANENT}

interface

uses WinTypes, WinProcs, OWindows, Strings;

{$I Version.Inc}

type
  PRibbonWindow = ^TRibbonWindow;
  TRibbonWindow = object(TWindow)
    GrayBrush: HBrush;
    GrayPen, WhitePen: HPen;
    Height, MoveableFlag : Integer;
    StudyFlag : word;
    HintText, CaptureText,
     RightButtonText, FixedText : array[0..80] of Char;
    Font: HFont;
    IsStockFont : Boolean;
    DefaultText : PChar;
    constructor Init(AParent: PWindowsObject);
    destructor Done; virtual;
    function GetClassName: PChar; virtual;
    procedure GetWindowClass(var WndClass: TWndClass); virtual;
    procedure Paint(PaintDC: hDC; var PS: TPaintStruct); virtual;
    procedure SetHelpText(Txt: PChar);
    procedure SetMoveableFlag( Flag : Integer);
    procedure SetStudyFlag( Flag : word);
    procedure WMSetText(var Msg: TMessage);
      virtual wm_First + wm_SetText;
  end;

implementation

constructor TRibbonWindow.Init(AParent: PWindowsObject);
var
  DC: hDC;
  OldFont: HFont;
  TM: TTExtMetric;
  LogFont: TLogFont;
const
  WVSRec: record
    MajorV,
    MinorV,
    RevChar: Word;
    Copr: PChar;
  end = (
    MajorV:0;
    MinorV:0;
    RevChar:0;
    Copr:' Copyright  1994');
begin
  { basic initialization for a non-MDI Window in an MDI Frame }
  Inherited Init(AParent, '');      { create the window normally }
  HintText[0] := #0;
  SetFlags(wb_MDIChild, False);     { turn off the MDI flag that TWindow set }
  DefaultProc := @DefWindowProc;    { and redirect the DefaultProc back }
  Attr.Style := ws_Border or ws_Child or ws_Visible;

  { establish Font for use in this window }
  Font := GetStockObject(ANSI_VAR_FONT);
  GetObject(Font,Sizeof(TLogFont), @LogFont);
  LogFont.lfWeight := 700;
  Font :=CreateFontIndirect(LogFont);
  IsStockFont := false;

  { Determine the height of this window }
  DC := GetDC(0);
  OldFont := SelectObject(DC, Font);
  GetTextMetrics(DC, TM);
  Height := TM.tmHeight + 8;
  SelectObject(DC, OldFont);
  ReleaseDC(0, DC);

  { Make the pens and brushes used to draw the status line }
  GrayPen := CreatePen(PS_SOLID,1,$00808080);
  WhitePen := CreatePen(PS_Solid,1,$00FFFFFF);
  GrayBrush := CreateSolidBrush($00C0C0C0);

  { Set up the text displayed in the hint line when there's nothing
    of greater importance to display }
   WVSrec.MajorV := MajorVersion;
   WVSRec.MinorV := MinorVersion;
   WVSRec.RevChar := Byte(RevisionChar);
   WVSPrintf(HintText,'Power Designer V%i.%#02i%c %s',WVSRec);
   DefaultText := StrNew(HintText);
   strcopy( RightButtonText,
            'Click the right mouse button to release capture');
   strcopy( CaptureText,
            'Release the left mouse button to release capture');
   strcopy( FixedText,
            'Can''t move components; check WorkSheet options');
   HintText[0] := #0;
   MoveableFlag:=0;
   StudyFlag:=0;
end;

destructor TRibbonWindow.Done;
begin
  DeleteObject(GrayPen);
  DeleteObject(WhitePen);
  StrDispose( DefaultText);
  { -- this object is still part of the class, so don't delete!!
  DeleteObject(GrayBrush);
  }
  if not IsStockFont then DeleteObject(Font);
  inherited Done;
end;

function TRibbonWindow.GetClassName;
begin
  GetClassName := 'StatusWindow';
end;

procedure TRibbonWindow.GetWindowClass(var WndClass: TWndClass);
begin
  inherited GetWindowClass(WndClass);
  WndClass.hbrBackGround := GrayBrush;
end;

procedure TRibbonWindow.Paint;
var
  OldFont: HFont;
  OldPen : hPen;
  R, R1, R2: TRect;
  Ph : array[0..8] of char;
  Ph2: array[0..10] of char;

  Procedure BoxIt( MyR : TRect);
  begin
  OldPen := SelectObject(PaintDC, GrayPen);
  MoveTo(PaintDC, MyR.left,MyR.bottom); { Rect de Moveable / Fixed }
  LineTo(PaintDC, MyR.left,MyR.top);
  LineTo(PaintDC, MyR.right,MyR.top);
  SelectObject(PaintDC, WhitePen);
  MoveTo(PaintDC, MyR.right,MyR.top); { Rect de Moveable / Fixed }
  LineTo(PaintDC, MyR.right,MyR.bottom);
  LineTo(PaintDC, MyR.left,MyR.bottom);

  end;

begin
  GetClientRect(hWindow,R);
  with R do
  begin
    Inc(Left,4); Inc(top,2); Dec(Right,4);
    Dec(Bottom,3);
    { Rect de Moveable / Fixed / Captured }
    R1.top:=top;
    R1.bottom:=bottom;
    R1.right:=right;
    R1.left:=right-70;
    { Rect de Study }
    R2.top:=top;
    R2.bottom:=bottom;
    R2.right:=R1.left-4;
    R2.left:=R2.right-70;

    Dec(right,148);
    SetBKMode(PaintDC, Transparent);
    BoxIt( R);
    BoxIt( R1);
    BoxIt( R2);
  end;
  case MoveableFlag of
          1   : strcopy( Ph, 'Moveable');
          2,5 : strcopy( Ph, 'Fixed');
          3,4 : strcopy( Ph, 'Captured');
          else strcopy( Ph, '')
  end;
  case StudyFlag of
          1   : strcopy( Ph2, 'Load Flow');
          2   : strcopy( Ph2, 'Trans Stab');
          else strcopy( Ph2, '')
  end;

  SelectObject(PaintDC, OldPen);
  SetTextColor(PaintDC, 0);
  OldFont := SelectObject(PaintDC, Font);
  with R do
         IntersectClipRect(PaintDC, left, top, right-2, bottom);
  Case MoveableFlag of
       3 : TextOut(PaintDC, 8, 3, RightButtonText, StrLen(RightButtonText));
       4 : TextOut(PaintDC, 8, 3, CaptureText, StrLen(CaptureText));
       5 : TextOut(PaintDC, 8, 3, FixedText, StrLen(FixedText));
       else
        begin
        if HintText[0] <> #0 then
                   TextOut(PaintDC, 8, 3, HintText, StrLen(HintText))
                             else
                   TextOut(PaintDC, 8, 3, DefaultText, StrLen(DefaultText));
        end;
  end;
  SelectClipRgn( PaintDC, 0);
  with R1 do
          IntersectClipRect(PaintDC, left, top, right-2, bottom);
  TextOut(PaintDC, R1.left+4, 3, Ph, StrLen(Ph));
  SelectClipRgn( PaintDC, 0);
  with R2 do
          IntersectClipRect(PaintDC, left, top, right-2, bottom);
  TextOut(PaintDC, R2.left+4, 3, Ph2, StrLen(Ph2));
  SelectObject(PaintDC,OldFont);
end;

procedure TRibbonWindow.SetHelpText;
begin
  if Txt = nil then
    HintText[0] := #0
  else
    StrLCopy(HintText, Txt, 80);
  if hWindow <> 0 then
    InvalidateRect(hWindow, nil, True);
end;

procedure TRibbonWindow.SetMoveableFlag( Flag : Integer);
begin
  MoveableFlag:=Flag;
  if hWindow <> 0 then InvalidateRect(hWindow, nil, True);
end;

procedure TRibbonWindow.SetStudyFlag( Flag : word);
begin
  StudyFlag:=Flag;
  if hWindow <> 0 then InvalidateRect(hWindow, nil, True);
end;

procedure TRibbonWindow.WMSetText;
begin
  SetHelpText(PChar(Msg.lPAram));
end;

End.
